@@ -182,6 +182,7 @@ enum old_files
   DEFAULT_OLD_FILES, /* default */
   UNLINK_FIRST_OLD_FILES, /* --unlink-first */
   KEEP_OLD_FILES, /* --keep-old-files */
+  OVERWRITE_OLD_DIRS, /* --overwrite-dir */
   OVERWRITE_OLD_FILES /* --overwrite */
 };
 GLOBAL enum old_files old_files_option;
@@ -345,7 +346,7 @@ extern FILE *stdlis;
 extern char *save_name;
 extern off_t save_sizeleft;
 extern off_t save_totsize;
-extern int write_archive_to_stdout;
+extern bool write_archive_to_stdout;
 
 size_t available_space_after PARAMS ((union block *));
 off_t current_block_ordinal PARAMS ((void));
@@ -421,6 +422,7 @@ enum read_header
 {
   HEADER_STILL_UNREAD,		/* for when read_header has not been called */
   HEADER_SUCCESS,		/* header successfully read and checksummed */
+  HEADER_SUCCESS_EXTENDED,	/* likewise, but we got an extended header */
   HEADER_ZERO_BLOCK,		/* zero block where header expected */
   HEADER_END_OF_FILE,		/* true end of file while header expected */
   HEADER_FAILURE		/* ill-formed header, or bad checksum */
@@ -461,7 +463,7 @@ void list_archive PARAMS ((void));
 void print_for_mkdir PARAMS ((char *, int, mode_t));
 void print_header PARAMS ((void));
 void read_and PARAMS ((void (*do_) ()));
-enum read_header read_header PARAMS ((void));
+enum read_header read_header PARAMS ((bool));
 void skip_file PARAMS ((off_t));
 void skip_member PARAMS ((void));
 
