@@ -1,5 +1,7 @@
 /* Delete entries from a tar archive.
-   Copyright (C) 1988, 92, 94, 96, 97, 2000 Free Software Foundation, Inc.
+
+   Copyright (C) 1988, 1992, 1994, 1996, 1997, 2000, 2001 Free
+   Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -21,15 +23,24 @@
 #include "rmt.h"
 
 static union block *new_record;
-static off_t records_read; 
 static int new_blocks;
-static int acting_as_filter;
+static bool acting_as_filter;
 
-/* FIXME: This module should not directly handle the following three
-   variables, instead, this should be done in buffer.c only.  */
+/* FIXME: This module should not directly handle the following
+   variables, instead, the interface should be cleaned up.  */
 extern union block *record_start;
 extern union block *record_end;
 extern union block *current_block;
+extern union block *recent_long_name;
+extern union block *recent_long_link;
+extern size_t recent_long_name_blocks;
+extern size_t recent_long_link_blocks;
+extern off_t records_read; 
+extern off_t records_written;
+
+/* The number of records skipped at the start of the archive, when
+   passing over members that are not deleted.  */
+static off_t records_skipped;
 
 /* Move archive descriptor by COUNT records worth.  If COUNT is
    positive we move forward, else we move negative.  If it's a tape,
@@ -38,6 +49,9 @@ extern union block *current_block;
 static void
 move_archive (off_t count)
 {
+  if (count == 0)
+    return;
+
 #ifdef MTIOCTOP
   {
     struct mtop operation;
@@ -91,7 +105,7 @@ write_record (int move_back_flag)
     }
   else
     {
-      move_archive (-(records_read + 1));
+      move_archive ((records_written + records_skipped) - records_read);
       flush_write ();
     }
 
@@ -102,14 +116,24 @@ write_record (int move_back_flag)
       /* Move the tape head back to where we were.  */
 
       if (! acting_as_filter)
-	move_archive (records_read);
-
-      records_read--;
+	move_archive (records_read - (records_written + records_skipped));
     }
 
   new_blocks = 0;
 }
 
+static void
+write_recent_blocks (union block *h, size_t blocks)
+{
+  size_t i;
+  for (i = 0; i < blocks; i++)
+    {
+      new_record[new_blocks++] = h[i];
+      if (new_blocks == blocking_factor)
+	write_record (1);
+    }
+}
+
 void
 delete_archive_members (void)
 {
@@ -128,7 +152,7 @@ delete_archive_members (void)
 
   do
     {
-      enum read_header status = read_header ();
+      enum read_header status = read_header (1);
 
       switch (status)
 	{
@@ -142,7 +166,9 @@ delete_archive_members (void)
 	      break;
 	    }
 	  name->found = 1;
-	  logical_status = HEADER_SUCCESS;
+	  /* Fall through.  */
+	case HEADER_SUCCESS_EXTENDED:
+	  logical_status = status;
 	  break;
 
 	case HEADER_ZERO_BLOCK:
@@ -182,9 +208,11 @@ delete_archive_members (void)
     }
   while (logical_status == HEADER_STILL_UNREAD);
 
+  records_skipped = records_read - 1;
   new_record = xmalloc (record_size);
 
-  if (logical_status == HEADER_SUCCESS)
+  if (logical_status == HEADER_SUCCESS
+      || logical_status == HEADER_SUCCESS_EXTENDED)
     {
       write_archive_to_stdout = 0;
 
@@ -194,12 +222,12 @@ delete_archive_members (void)
       if (new_blocks)
 	memcpy (new_record, record_start, new_blocks * BLOCKSIZE);
 
-#if 0
-      /* FIXME: Old code, before the goto was inserted.  To be redesigned.  */
-      skip_member ();
-#endif
-      logical_status = HEADER_STILL_UNREAD;
-      goto flush_file;
+      if (logical_status == HEADER_SUCCESS)
+	{
+	  /* FIXME: Pheew!  This is crufty code!  */
+	  logical_status = HEADER_STILL_UNREAD;
+	  goto flush_file;
+	}
 
       /* FIXME: Solaris 2.4 Sun cc (the ANSI one, not the old K&R) says:
 	 "delete.c", line 223: warning: loop not entered at top
@@ -211,11 +239,8 @@ delete_archive_members (void)
 	  /* Fill in a record.  */
 
 	  if (current_block == record_end)
-	    {
-	      flush_archive ();
-	      records_read++;
-	    }
-	  status = read_header ();
+	    flush_archive ();
+	  status = read_header (0);
 
 	  if (status == HEADER_ZERO_BLOCK && ignore_zeros_option)
 	    {
@@ -248,7 +273,6 @@ delete_archive_members (void)
 		{
 		  blocks_to_skip -= (record_end - current_block);
 		  flush_archive ();
-		  records_read++;
 		}
 	      current_block += blocks_to_skip;
 	      blocks_to_skip = 0;
@@ -257,6 +281,8 @@ delete_archive_members (void)
 
 	  /* Copy header.  */
 
+	  write_recent_blocks (recent_long_name, recent_long_name_blocks);
+	  write_recent_blocks (recent_long_link, recent_long_link_blocks);
 	  new_record[new_blocks] = *current_header;
 	  new_blocks++;
 	  blocks_to_keep
@@ -278,7 +304,6 @@ delete_archive_members (void)
 	      if (current_block == record_end)
 		{
 		  flush_read ();
-		  records_read++;
 		  current_block = record_start;
 		  kept_blocks_in_record = blocking_factor;
 		  if (kept_blocks_in_record > blocks_to_keep)
