@@ -1,5 +1,7 @@
 /* Miscellaneous functions, not really specific to GNU tar.
-   Copyright 1988,92,94,95,96,97,99,2000, 2001 Free Software Foundation, Inc.
+
+   Copyright 1988, 1992, 1994, 1995, 1996, 1997, 1999, 2000, 2001 Free
+   Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -585,6 +587,14 @@ exec_fatal (char const *name)
   call_arg_fatal ("exec", name);
 }
 
+void
+link_error (char const *target, char const *source)
+{
+  int e = errno;
+  ERROR ((0, e, _("%s: Cannot hard link to %s"),
+	  quotearg_colon (source), quote_n (1, target)));
+}
+
 void
 mkdir_error (char const *name)
 {
